@@ -19,7 +19,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id$
+ * $Id: CXvidDecoder.cpp,v 1.16 2005/10/08 00:58:02 suxen_drol Exp $
  *
  ****************************************************************************/
 
@@ -36,11 +36,11 @@
 	place these paths at the top of the Tools|Options|Directories list
 
 	headers:
-	C:\DXVCSDK\include
-	C:\DXVCSDK\samples\Multimedia\DirectShow\BaseClasses
+	C:\DX90SDK\Include
+	C:\DX90SDK\Samples\C++\DirectShow\BaseClasses
 	
-	libraries (optional):
-	C:\DXVCSDK\samples\Multimedia\DirectShow\BaseClasses\Release
+	C:\DX90SDK\Samples\C++\DirectShow\BaseClasses\Release
+	C:\DX90SDK\Samples\C++\DirectShow\BaseClasses\Debug
 */
 
 
@@ -82,6 +82,7 @@ const AMOVIESETUP_MEDIATYPE sudInputPinTypes[] =
 	{ &MEDIATYPE_Video, &CLSID_DX50 },
 	{ &MEDIATYPE_Video, &CLSID_DX50_UC },
 	{ &MEDIATYPE_Video, &CLSID_MP4V },
+  { &MEDIATYPE_Video, &CLSID_MP4V_UC },
 };
 
 const AMOVIESETUP_MEDIATYPE sudOutputPinTypes[] =
@@ -204,39 +205,61 @@ STDMETHODIMP CXvidDecoder::NonDelegatingQueryInterface(REFIID riid, void **ppv)
 #define XVID_DLL_NAME "xvidcore.dll"
 
 CXvidDecoder::CXvidDecoder(LPUNKNOWN punk, HRESULT *phr) :
-    CVideoTransformFilter(NAME("CXvidDecoder"), punk, CLSID_XVID)
+    CVideoTransformFilter(NAME("CXvidDecoder"), punk, CLSID_XVID), m_hdll (NULL)
 {
 	DPRINTF("Constructor");
 
+    xvid_decore_func = NULL; // Hmm, some strange errors appearing if I try to initialize...
+    xvid_global_func = NULL; // ...this in constructor's init-list. So, they assigned here.
+
+    LoadRegistryInfo();
+
+    *phr = OpenLib();
+}
+
+HRESULT CXvidDecoder::OpenLib()
+{
+    DPRINTF("OpenLib");
+
+    if (m_hdll != NULL)
+		return E_UNEXPECTED; // Seems, that library already opened.
+
 	xvid_gbl_init_t init;
 	memset(&init, 0, sizeof(init));
 	init.version = XVID_VERSION;
 
-	ar_x = ar_y = 0;
-
 	m_hdll = LoadLibrary(XVID_DLL_NAME);
 	if (m_hdll == NULL) {
 		DPRINTF("dll load failed");
-		MessageBox(0, XVID_DLL_NAME " not found","Error", 0);
-		return;
+		MessageBox(0, XVID_DLL_NAME " not found","Error", MB_TOPMOST);
+		return E_FAIL;
 	}
 
 	xvid_global_func = (int (__cdecl *)(void *, int, void *, void *))GetProcAddress(m_hdll, "xvid_global");
 	if (xvid_global_func == NULL) {
-		MessageBox(0, "xvid_global() not found", "Error", 0);
-		return;
+        FreeLibrary(m_hdll);
+        m_hdll = NULL;
+		MessageBox(0, "xvid_global() not found", "Error", MB_TOPMOST);
+		return E_FAIL;
 	}
 
 	xvid_decore_func = (int (__cdecl *)(void *, int, void *, void *))GetProcAddress(m_hdll, "xvid_decore");
 	if (xvid_decore_func == NULL) {
-		MessageBox(0, "xvid_decore() not found", "Error", 0);
-		return;
+        xvid_global_func = NULL;
+        FreeLibrary(m_hdll);
+        m_hdll = NULL;
+		MessageBox(0, "xvid_decore() not found", "Error", MB_TOPMOST);
+		return E_FAIL;
 	}
 
 	if (xvid_global_func(0, XVID_GBL_INIT, &init, NULL) < 0)
 	{
-		MessageBox(0, "xvid_global() failed", "Error", 0);
-		return;
+        xvid_global_func = NULL;
+        xvid_decore_func = NULL;
+        FreeLibrary(m_hdll);
+        m_hdll = NULL;
+		MessageBox(0, "xvid_global() failed", "Error", MB_TOPMOST);
+		return E_FAIL;
 	}
 
 	memset(&m_create, 0, sizeof(m_create));
@@ -246,8 +269,6 @@ CXvidDecoder::CXvidDecoder(LPUNKNOWN punk, HRESULT *phr) :
 	memset(&m_frame, 0, sizeof(m_frame));
 	m_frame.version = XVID_VERSION;
 
-	LoadRegistryInfo();
-
 	USE_IYUV = false;
 	USE_YV12 = false;
 	USE_YUY2 = false;
@@ -285,13 +306,35 @@ CXvidDecoder::CXvidDecoder(LPUNKNOWN punk, HRESULT *phr) :
 		USE_RGB32 = true;
 		break;
 	}
+
+	switch (g_config.aspect_ratio)
+	{
+	case 0:
+	case 1:
+		break;
+	case 2:
+		ar_x = 4;
+		ar_y = 3;
+		break;
+	case 3:
+		ar_x = 16;
+		ar_y = 9;
+		break;
+	case 4:
+		ar_x = 47;
+		ar_y = 20;
+		break;
+	}
+	
+	return S_OK;
 }
 
 void CXvidDecoder::CloseLib()
 {
-	DPRINTF("Destructor");
+	DPRINTF("CloseLib");
 
-	if (m_create.handle != NULL) {
+	if ((m_create.handle != NULL) && (xvid_decore_func != NULL))
+	{
 		xvid_decore_func(m_create.handle, XVID_DEC_DESTROY, 0, 0);
 		m_create.handle = NULL;
 	}
@@ -300,12 +343,15 @@ void CXvidDecoder::CloseLib()
 		FreeLibrary(m_hdll);
 		m_hdll = NULL;
 	}
+    xvid_decore_func = NULL;
+    xvid_global_func = NULL;
 }
 
 /* destructor */
 
 CXvidDecoder::~CXvidDecoder()
 {
+    DPRINTF("Destructor");
 	CloseLib();
 }
 
@@ -317,6 +363,8 @@ HRESULT CXvidDecoder::CheckInputType(const CMediaType * mtIn)
 {
 	DPRINTF("CheckInputType");
 	BITMAPINFOHEADER * hdr;
+
+	ar_x = ar_y = 0;
 	
 	if (*mtIn->Type() != MEDIATYPE_Video)
 	{
@@ -325,25 +373,71 @@ HRESULT CXvidDecoder::CheckInputType(const CMediaType * mtIn)
 		return VFW_E_TYPE_NOT_ACCEPTED;
 	}
 
+    if (m_hdll == NULL)
+    {
+		HRESULT hr = OpenLib();
+
+        if (FAILED(hr) || (m_hdll == NULL)) // Paranoid checks.
+			return VFW_E_TYPE_NOT_ACCEPTED;
+    }
+
 	if (*mtIn->FormatType() == FORMAT_VideoInfo)
 	{
 		VIDEOINFOHEADER * vih = (VIDEOINFOHEADER *) mtIn->Format();
 		hdr = &vih->bmiHeader;
-		/* PAR (x:y) is (1/ppm_X):(1/ppm_Y) where ppm is pixels-per-meter
-		   which is equal to ppm_Y:ppm_X */
-		ar_x = vih->bmiHeader.biYPelsPerMeter * abs(hdr->biWidth);
-		ar_y = vih->bmiHeader.biXPelsPerMeter * abs(hdr->biHeight);
-		DPRINTF("VIDEOINFOHEADER PAR: %d:%d -> AR %d:%d",
-			vih->bmiHeader.biYPelsPerMeter,vih->bmiHeader.biXPelsPerMeter, ar_x, ar_y);
 	}
 	else if (*mtIn->FormatType() == FORMAT_VideoInfo2)
 	{
 		VIDEOINFOHEADER2 * vih2 = (VIDEOINFOHEADER2 *) mtIn->Format();
 		hdr = &vih2->bmiHeader;
-		ar_x = vih2->dwPictAspectRatioX;
-		ar_y = vih2->dwPictAspectRatioY;
+		if (g_config.aspect_ratio == 0 || g_config.aspect_ratio == 1) {
+			ar_x = vih2->dwPictAspectRatioX;
+			ar_y = vih2->dwPictAspectRatioY;
+		}
 		DPRINTF("VIDEOINFOHEADER2 AR: %d:%d", ar_x, ar_y);
 	}
+  else if (*mtIn->FormatType() == FORMAT_MPEG2Video) {
+    MPEG2VIDEOINFO * mpgvi = (MPEG2VIDEOINFO*)mtIn->Format();
+    VIDEOINFOHEADER2 * vih2 = &mpgvi->hdr;
+		hdr = &vih2->bmiHeader;
+		if (g_config.aspect_ratio == 0 || g_config.aspect_ratio == 1) {
+			ar_x = vih2->dwPictAspectRatioX;
+			ar_y = vih2->dwPictAspectRatioY;
+		}
+		DPRINTF("VIDEOINFOHEADER2 AR: %d:%d", ar_x, ar_y);
+
+    /* haali media splitter reports VOL information in the format header */
+
+    if (mpgvi->cbSequenceHeader>0) {
+
+      xvid_dec_stats_t stats;
+	    memset(&stats, 0, sizeof(stats));
+	    stats.version = XVID_VERSION;
+
+	    if (m_create.handle == NULL) {
+		    if (xvid_decore_func == NULL)
+			    return E_FAIL;
+		    if (xvid_decore_func(0, XVID_DEC_CREATE, &m_create, 0) < 0) {
+          DPRINTF("*** XVID_DEC_CREATE error");
+			    return S_FALSE;
+		    }
+	    }
+
+      m_frame.general = 0;
+      m_frame.bitstream = (void*)mpgvi->dwSequenceHeader;
+      m_frame.length = mpgvi->cbSequenceHeader;
+      m_frame.output.csp = XVID_CSP_NULL;
+
+      if (xvid_decore_func(m_create.handle, XVID_DEC_DECODE, &m_frame, &stats) >= 0) {
+        /* honour video dimensions reported in VOL header */
+	      if (stats.type == XVID_TYPE_VOL) {
+          hdr->biWidth = stats.data.vol.width;
+          hdr->biHeight = stats.data.vol.height;
+        }
+      }
+
+    }
+  }
 	else
 	{
 		DPRINTF("Error: Unknown FormatType");
@@ -361,7 +455,7 @@ HRESULT CXvidDecoder::CheckInputType(const CMediaType * mtIn)
 
 	switch(hdr->biCompression)
 	{
-
+  case FOURCC_mp4v:
 	case FOURCC_MP4V:
 		if (!(g_config.supported_4cc & SUPPORT_MP4V)) {
 			CloseLib();
@@ -421,11 +515,12 @@ HRESULT CXvidDecoder::GetMediaType(int iPosition, CMediaType *mtOut)
 		if (ar_x != 0 && ar_y != 0) {
 			vih->dwPictAspectRatioX = ar_x;
 			vih->dwPictAspectRatioY = ar_y;
+			forced_ar = true;
 		} else { // just to be safe
 			vih->dwPictAspectRatioX = m_create.width;
 			vih->dwPictAspectRatioY = abs(m_create.height);
-		}
-
+			forced_ar = false;
+		} 
 	} else {
 
 		VIDEOINFOHEADER * vih = (VIDEOINFOHEADER *) mtOut->ReallocFormatBuffer(sizeof(VIDEOINFOHEADER));
@@ -694,6 +789,9 @@ HRESULT CXvidDecoder::Transform(IMediaSample *pIn, IMediaSample *pOut)
 
 	if (m_create.handle == NULL)
 	{
+		if (xvid_decore_func == NULL)
+			return E_FAIL;
+
 		if (xvid_decore_func(0, XVID_DEC_CREATE, &m_create, 0) < 0)
 		{
             DPRINTF("*** XVID_DEC_CREATE error");
@@ -738,16 +836,27 @@ HRESULT CXvidDecoder::Transform(IMediaSample *pIn, IMediaSample *pOut)
 
 	if (g_config.nDeblock_UV)
 		m_frame.general |= XVID_DEBLOCKUV;
-/*
-	if (g_config.nDering)
-		m_frame.general |= XVID_DERING;
-*/
+
+	if (g_config.nDering_Y)
+		m_frame.general |= XVID_DERINGY;
+
+	if (g_config.nDering_UV)
+		m_frame.general |= XVID_DERINGUV;
+
 	if (g_config.nFilmEffect)
 		m_frame.general |= XVID_FILMEFFECT;
 
+	m_frame.brightness = g_config.nBrightness;
+
 	m_frame.output.csp &= ~XVID_CSP_VFLIP;
 	m_frame.output.csp |= rgb_flip^(g_config.nFlipVideo ? XVID_CSP_VFLIP : 0);
 
+    // Paranoid check.
+    if (xvid_decore_func == NULL)
+		return E_FAIL;
+
+
+
 repeat :
 
 	if (pIn->IsPreroll() != S_OK)
@@ -758,6 +867,30 @@ repeat :
 		{
             DPRINTF("*** XVID_DEC_DECODE");
 			return S_FALSE;
+		} else 
+			if (g_config.aspect_ratio == 0 || g_config.aspect_ratio == 1 && forced_ar == false) {
+        
+      if (stats.type != XVID_TYPE_NOTHING) {  /* dont attempt to set vmr aspect ratio if no frame was returned by decoder */
+			// inspired by minolta! works for VMR 7 + 9
+			IMediaSample2 *pOut2 = NULL;
+			AM_SAMPLE2_PROPERTIES outProp2;
+			if (SUCCEEDED(pOut->QueryInterface(IID_IMediaSample2, (void **)&pOut2)) &&
+				SUCCEEDED(pOut2->GetProperties(FIELD_OFFSET(AM_SAMPLE2_PROPERTIES, tStart), (PBYTE)&outProp2)))
+			{
+				CMediaType mtOut2 = m_pOutput->CurrentMediaType();
+				VIDEOINFOHEADER2* vihOut2 = (VIDEOINFOHEADER2*)mtOut2.Format();
+
+				if (*mtOut2.FormatType() == FORMAT_VideoInfo2 && 
+					vihOut2->dwPictAspectRatioX != ar_x && vihOut2->dwPictAspectRatioY != ar_y)
+				{
+					vihOut2->dwPictAspectRatioX = ar_x;
+					vihOut2->dwPictAspectRatioY = ar_y;
+					pOut2->SetMediaType(&mtOut2);
+					m_pOutput->SetMediaType(&mtOut2);
+				}
+				pOut2->Release();
+			}
+      }
 		}
 	}
 	else
@@ -770,7 +903,7 @@ repeat :
 		/* Disable postprocessing to speed-up seeking */
 		m_frame.general &= ~XVID_DEBLOCKY;
 		m_frame.general &= ~XVID_DEBLOCKUV;
-/*		m_frame.general &= ~XVID_DERING; */
+		/*m_frame.general &= ~XVID_DERING;*/
 		m_frame.general &= ~XVID_FILMEFFECT;
 
 		length = xvid_decore_func(m_create.handle, XVID_DEC_DECODE, &m_frame, &stats);
@@ -798,10 +931,23 @@ repeat :
 			DPRINTF("TODO: auto-resize");
 			return S_FALSE;
 		}
-		
-//		pOut->SetDiscontinuity(TRUE);
+
 		pOut->SetSyncPoint(TRUE);
 
+		if (g_config.aspect_ratio == 0 || g_config.aspect_ratio == 1) { /* auto */
+			int par_x, par_y;
+			if (stats.data.vol.par == XVID_PAR_EXT) {
+				par_x = stats.data.vol.par_width;
+				par_y = stats.data.vol.par_height;
+			} else {
+				par_x = PARS[stats.data.vol.par-1][0];
+				par_y = PARS[stats.data.vol.par-1][1];
+			}
+
+			ar_x = par_x * stats.data.vol.width;
+			ar_y = par_y * stats.data.vol.height;
+		}
+
 		m_frame.bitstream = (BYTE*)m_frame.bitstream + length;
 		m_frame.length -= length;
 		goto repeat;
